# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: wandb/proto/wandb_internal.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='wandb/proto/wandb_internal.proto',
  package='wandb_internal',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n wandb/proto/wandb_internal.proto\x12\x0ewandb_internal\x1a\x1fgoogle/protobuf/timestamp.proto\"\xb7\x04\n\x06Record\x12\x0b\n\x03num\x18\x01 \x01(\x03\x12.\n\x07history\x18\x02 \x01(\x0b\x32\x1b.wandb_internal.HistoryDataH\x00\x12.\n\x07summary\x18\x03 \x01(\x0b\x32\x1b.wandb_internal.SummaryDataH\x00\x12,\n\x06output\x18\x04 \x01(\x0b\x32\x1a.wandb_internal.OutputDataH\x00\x12,\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x1a.wandb_internal.ConfigDataH\x00\x12*\n\x05\x66iles\x18\x06 \x01(\x0b\x32\x19.wandb_internal.FilesDataH\x00\x12*\n\x05stats\x18\x07 \x01(\x0b\x32\x19.wandb_internal.StatsDataH\x00\x12\x30\n\x08\x61rtifact\x18\x08 \x01(\x0b\x32\x1c.wandb_internal.ArtifactDataH\x00\x12&\n\x03run\x18\x11 \x01(\x0b\x32\x17.wandb_internal.RunDataH\x00\x12(\n\x04\x65xit\x18\x12 \x01(\x0b\x32\x18.wandb_internal.ExitDataH\x00\x12,\n\x06status\x18\x13 \x01(\x0b\x32\x1a.wandb_internal.StatusDataH\x00\x12(\n\x06tbdata\x18\x14 \x01(\x0b\x32\x16.wandb_internal.TBDataH\x00\x12(\n\x07\x63ontrol\x18\x10 \x01(\x0b\x32\x17.wandb_internal.ControlB\x06\n\x04\x64\x61ta\"*\n\x07\x43ontrol\x12\x10\n\x08req_resp\x18\x01 \x01(\x08\x12\r\n\x05local\x18\x02 \x01(\x08\"\x80\x03\n\x07RunData\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x0e\n\x06\x65ntity\x18\x02 \x01(\t\x12\x0f\n\x07project\x18\x03 \x01(\t\x12*\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x1a.wandb_internal.ConfigData\x12,\n\x07summary\x18\x05 \x01(\x0b\x32\x1b.wandb_internal.SummaryData\x12\x11\n\trun_group\x18\x06 \x01(\t\x12\x10\n\x08job_type\x18\x07 \x01(\t\x12\x14\n\x0c\x64isplay_name\x18\x08 \x01(\t\x12\r\n\x05notes\x18\t \x01(\t\x12\x0c\n\x04tags\x18\n \x03(\t\x12.\n\x08settings\x18\x0b \x01(\x0b\x32\x1c.wandb_internal.SettingsData\x12\x10\n\x08sweep_id\x18\x0c \x01(\t\x12\x0c\n\x04host\x18\r \x01(\t\x12\x12\n\nstorage_id\x18\x10 \x01(\t\x12.\n\nstart_time\x18\x11 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\":\n\x0cSettingsData\x12*\n\x04item\x18\x01 \x03(\x0b\x32\x1c.wandb_internal.SettingsItem\"/\n\x0cSettingsItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\nvalue_json\x18\x10 \x01(\t\"8\n\x0bHistoryData\x12)\n\x04item\x18\x01 \x03(\x0b\x32\x1b.wandb_internal.HistoryItem\"B\n\x0bHistoryItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\nnested_key\x18\x02 \x03(\t\x12\x12\n\nvalue_json\x18\x10 \x01(\t\"\xab\x01\n\nOutputData\x12:\n\x0boutput_type\x18\x01 \x01(\x0e\x32%.wandb_internal.OutputData.OutputType\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04line\x18\x03 \x01(\t\"$\n\nOutputType\x12\n\n\x06STDERR\x10\x00\x12\n\n\x06STDOUT\x10\x01\"d\n\nConfigData\x12*\n\x06update\x18\x01 \x03(\x0b\x32\x1a.wandb_internal.ConfigItem\x12*\n\x06remove\x18\x02 \x03(\x0b\x32\x1a.wandb_internal.ConfigItem\"A\n\nConfigItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\nnested_key\x18\x02 \x03(\t\x12\x12\n\nvalue_json\x18\x10 \x01(\t\"g\n\x0bSummaryData\x12+\n\x06update\x18\x01 \x03(\x0b\x32\x1b.wandb_internal.SummaryItem\x12+\n\x06remove\x18\x02 \x03(\x0b\x32\x1b.wandb_internal.SummaryItem\"B\n\x0bSummaryItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\nnested_key\x18\x02 \x03(\t\x12\x12\n\nvalue_json\x18\x10 \x01(\t\"5\n\tFilesData\x12(\n\x05\x66iles\x18\x01 \x03(\x0b\x32\x19.wandb_internal.FilesItem\"\x90\x01\n\tFilesItem\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x34\n\x06policy\x18\x02 \x01(\x0e\x32$.wandb_internal.FilesItem.PolicyType\x12\x15\n\rexternal_path\x18\x10 \x01(\t\"(\n\nPolicyType\x12\x07\n\x03NOW\x10\x00\x12\x07\n\x03\x45ND\x10\x01\x12\x08\n\x04LIVE\x10\x02\"\xb5\x01\n\tStatsData\x12\x37\n\nstats_type\x18\x01 \x01(\x0e\x32#.wandb_internal.StatsData.StatsType\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x04item\x18\x03 \x03(\x0b\x32\x19.wandb_internal.StatsItem\"\x17\n\tStatsType\x12\n\n\x06SYSTEM\x10\x00\",\n\tStatsItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\nvalue_json\x18\x10 \x01(\t\"\x87\x02\n\x0c\x41rtifactData\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x65ntity\x18\x03 \x01(\t\x12\x0c\n\x04type\x18\x04 \x01(\t\x12\x0c\n\x04name\x18\x05 \x01(\t\x12\x0e\n\x06\x64igest\x18\x06 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x07 \x01(\t\x12\x10\n\x08metadata\x18\x08 \x01(\t\x12\x14\n\x0cuser_created\x18\t \x01(\x08\x12\x18\n\x10use_after_commit\x18\n \x01(\x08\x12\x0f\n\x07\x61liases\x18\x0b \x03(\t\x12\x32\n\x08manifest\x18\x0c \x01(\x0b\x32 .wandb_internal.ArtifactManifest\"\xbc\x01\n\x10\x41rtifactManifest\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x16\n\x0estorage_policy\x18\x02 \x01(\t\x12\x46\n\x15storage_policy_config\x18\x03 \x03(\x0b\x32\'.wandb_internal.StoragePolicyConfigItem\x12\x37\n\x08\x63ontents\x18\x04 \x03(\x0b\x32%.wandb_internal.ArtifactManifestEntry\"\xa0\x01\n\x15\x41rtifactManifestEntry\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06\x64igest\x18\x02 \x01(\t\x12\x0b\n\x03ref\x18\x03 \x01(\t\x12\x0c\n\x04size\x18\x04 \x01(\x03\x12\x10\n\x08mimetype\x18\x05 \x01(\t\x12\x12\n\nlocal_path\x18\x06 \x01(\t\x12(\n\x05\x65xtra\x18\x10 \x03(\x0b\x32\x19.wandb_internal.ExtraItem\",\n\tExtraItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\nvalue_json\x18\x02 \x01(\t\":\n\x17StoragePolicyConfigItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\nvalue_json\x18\x02 \x01(\t\"\x1d\n\x08\x45xitData\x12\x11\n\texit_code\x18\x01 \x01(\x05\"\x0c\n\nStatusData\"\'\n\x06TBData\x12\x0f\n\x07log_dir\x18\x01 \x01(\t\x12\x0c\n\x04save\x18\x02 \x01(\x08\"7\n\x0fRunUpdateResult\x12$\n\x03run\x18\x01 \x01(\x0b\x32\x17.wandb_internal.RunData\"\x0f\n\rRunExitResult\"\x0b\n\tLogResult\"\x0e\n\x0cStatusResult\"\x0f\n\rSummaryResult\"\x0e\n\x0cOutputResult\"\x0e\n\x0c\x43onfigResult\"\x92\x03\n\x0cResultRecord\x12\x35\n\nrun_result\x18\x11 \x01(\x0b\x32\x1f.wandb_internal.RunUpdateResultH\x00\x12\x34\n\x0b\x65xit_result\x18\x12 \x01(\x0b\x32\x1d.wandb_internal.RunExitResultH\x00\x12\x35\n\rstatus_result\x18\x13 \x01(\x0b\x32\x1c.wandb_internal.StatusResultH\x00\x12/\n\nlog_result\x18\x14 \x01(\x0b\x32\x19.wandb_internal.LogResultH\x00\x12\x37\n\x0esummary_result\x18\x15 \x01(\x0b\x32\x1d.wandb_internal.SummaryResultH\x00\x12\x35\n\routput_result\x18\x16 \x01(\x0b\x32\x1c.wandb_internal.OutputResultH\x00\x12\x35\n\rconfig_result\x18\x17 \x01(\x0b\x32\x1c.wandb_internal.ConfigResultH\x00\x42\x06\n\x04\x64\x61tab\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])



_OUTPUTDATA_OUTPUTTYPE = _descriptor.EnumDescriptor(
  name='OutputType',
  full_name='wandb_internal.OutputData.OutputType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STDERR', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STDOUT', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1457,
  serialized_end=1493,
)
_sym_db.RegisterEnumDescriptor(_OUTPUTDATA_OUTPUTTYPE)

_FILESITEM_POLICYTYPE = _descriptor.EnumDescriptor(
  name='PolicyType',
  full_name='wandb_internal.FilesItem.PolicyType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOW', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='END', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LIVE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1997,
  serialized_end=2037,
)
_sym_db.RegisterEnumDescriptor(_FILESITEM_POLICYTYPE)

_STATSDATA_STATSTYPE = _descriptor.EnumDescriptor(
  name='StatsType',
  full_name='wandb_internal.StatsData.StatsType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SYSTEM', index=0, number=0,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2198,
  serialized_end=2221,
)
_sym_db.RegisterEnumDescriptor(_STATSDATA_STATSTYPE)


_RECORD = _descriptor.Descriptor(
  name='Record',
  full_name='wandb_internal.Record',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='num', full_name='wandb_internal.Record.num', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='history', full_name='wandb_internal.Record.history', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='summary', full_name='wandb_internal.Record.summary', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='output', full_name='wandb_internal.Record.output', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='wandb_internal.Record.config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='files', full_name='wandb_internal.Record.files', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stats', full_name='wandb_internal.Record.stats', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='artifact', full_name='wandb_internal.Record.artifact', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run', full_name='wandb_internal.Record.run', index=8,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exit', full_name='wandb_internal.Record.exit', index=9,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='wandb_internal.Record.status', index=10,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tbdata', full_name='wandb_internal.Record.tbdata', index=11,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='control', full_name='wandb_internal.Record.control', index=12,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='wandb_internal.Record.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=86,
  serialized_end=653,
)


_CONTROL = _descriptor.Descriptor(
  name='Control',
  full_name='wandb_internal.Control',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='req_resp', full_name='wandb_internal.Control.req_resp', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='local', full_name='wandb_internal.Control.local', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=655,
  serialized_end=697,
)


_RUNDATA = _descriptor.Descriptor(
  name='RunData',
  full_name='wandb_internal.RunData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run_id', full_name='wandb_internal.RunData.run_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entity', full_name='wandb_internal.RunData.entity', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='project', full_name='wandb_internal.RunData.project', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='wandb_internal.RunData.config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='summary', full_name='wandb_internal.RunData.summary', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_group', full_name='wandb_internal.RunData.run_group', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='job_type', full_name='wandb_internal.RunData.job_type', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='display_name', full_name='wandb_internal.RunData.display_name', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='notes', full_name='wandb_internal.RunData.notes', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='wandb_internal.RunData.tags', index=9,
      number=10, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='settings', full_name='wandb_internal.RunData.settings', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sweep_id', full_name='wandb_internal.RunData.sweep_id', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host', full_name='wandb_internal.RunData.host', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='storage_id', full_name='wandb_internal.RunData.storage_id', index=13,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='wandb_internal.RunData.start_time', index=14,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=700,
  serialized_end=1084,
)


_SETTINGSDATA = _descriptor.Descriptor(
  name='SettingsData',
  full_name='wandb_internal.SettingsData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='item', full_name='wandb_internal.SettingsData.item', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1086,
  serialized_end=1144,
)


_SETTINGSITEM = _descriptor.Descriptor(
  name='SettingsItem',
  full_name='wandb_internal.SettingsItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='wandb_internal.SettingsItem.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_json', full_name='wandb_internal.SettingsItem.value_json', index=1,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1146,
  serialized_end=1193,
)


_HISTORYDATA = _descriptor.Descriptor(
  name='HistoryData',
  full_name='wandb_internal.HistoryData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='item', full_name='wandb_internal.HistoryData.item', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1195,
  serialized_end=1251,
)


_HISTORYITEM = _descriptor.Descriptor(
  name='HistoryItem',
  full_name='wandb_internal.HistoryItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='wandb_internal.HistoryItem.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nested_key', full_name='wandb_internal.HistoryItem.nested_key', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_json', full_name='wandb_internal.HistoryItem.value_json', index=2,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1253,
  serialized_end=1319,
)


_OUTPUTDATA = _descriptor.Descriptor(
  name='OutputData',
  full_name='wandb_internal.OutputData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='output_type', full_name='wandb_internal.OutputData.output_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='wandb_internal.OutputData.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='line', full_name='wandb_internal.OutputData.line', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OUTPUTDATA_OUTPUTTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1322,
  serialized_end=1493,
)


_CONFIGDATA = _descriptor.Descriptor(
  name='ConfigData',
  full_name='wandb_internal.ConfigData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='update', full_name='wandb_internal.ConfigData.update', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='remove', full_name='wandb_internal.ConfigData.remove', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1495,
  serialized_end=1595,
)


_CONFIGITEM = _descriptor.Descriptor(
  name='ConfigItem',
  full_name='wandb_internal.ConfigItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='wandb_internal.ConfigItem.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nested_key', full_name='wandb_internal.ConfigItem.nested_key', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_json', full_name='wandb_internal.ConfigItem.value_json', index=2,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1597,
  serialized_end=1662,
)


_SUMMARYDATA = _descriptor.Descriptor(
  name='SummaryData',
  full_name='wandb_internal.SummaryData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='update', full_name='wandb_internal.SummaryData.update', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='remove', full_name='wandb_internal.SummaryData.remove', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1664,
  serialized_end=1767,
)


_SUMMARYITEM = _descriptor.Descriptor(
  name='SummaryItem',
  full_name='wandb_internal.SummaryItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='wandb_internal.SummaryItem.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nested_key', full_name='wandb_internal.SummaryItem.nested_key', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_json', full_name='wandb_internal.SummaryItem.value_json', index=2,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1769,
  serialized_end=1835,
)


_FILESDATA = _descriptor.Descriptor(
  name='FilesData',
  full_name='wandb_internal.FilesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='files', full_name='wandb_internal.FilesData.files', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1837,
  serialized_end=1890,
)


_FILESITEM = _descriptor.Descriptor(
  name='FilesItem',
  full_name='wandb_internal.FilesItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='wandb_internal.FilesItem.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='policy', full_name='wandb_internal.FilesItem.policy', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='external_path', full_name='wandb_internal.FilesItem.external_path', index=2,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FILESITEM_POLICYTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1893,
  serialized_end=2037,
)


_STATSDATA = _descriptor.Descriptor(
  name='StatsData',
  full_name='wandb_internal.StatsData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stats_type', full_name='wandb_internal.StatsData.stats_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='wandb_internal.StatsData.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='item', full_name='wandb_internal.StatsData.item', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _STATSDATA_STATSTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2040,
  serialized_end=2221,
)


_STATSITEM = _descriptor.Descriptor(
  name='StatsItem',
  full_name='wandb_internal.StatsItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='wandb_internal.StatsItem.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_json', full_name='wandb_internal.StatsItem.value_json', index=1,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2223,
  serialized_end=2267,
)


_ARTIFACTDATA = _descriptor.Descriptor(
  name='ArtifactData',
  full_name='wandb_internal.ArtifactData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run_id', full_name='wandb_internal.ArtifactData.run_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='project', full_name='wandb_internal.ArtifactData.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entity', full_name='wandb_internal.ArtifactData.entity', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='wandb_internal.ArtifactData.type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='wandb_internal.ArtifactData.name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='digest', full_name='wandb_internal.ArtifactData.digest', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='wandb_internal.ArtifactData.description', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='wandb_internal.ArtifactData.metadata', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_created', full_name='wandb_internal.ArtifactData.user_created', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='use_after_commit', full_name='wandb_internal.ArtifactData.use_after_commit', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='aliases', full_name='wandb_internal.ArtifactData.aliases', index=10,
      number=11, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='manifest', full_name='wandb_internal.ArtifactData.manifest', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2270,
  serialized_end=2533,
)


_ARTIFACTMANIFEST = _descriptor.Descriptor(
  name='ArtifactManifest',
  full_name='wandb_internal.ArtifactManifest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='wandb_internal.ArtifactManifest.version', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='storage_policy', full_name='wandb_internal.ArtifactManifest.storage_policy', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='storage_policy_config', full_name='wandb_internal.ArtifactManifest.storage_policy_config', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contents', full_name='wandb_internal.ArtifactManifest.contents', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2536,
  serialized_end=2724,
)


_ARTIFACTMANIFESTENTRY = _descriptor.Descriptor(
  name='ArtifactManifestEntry',
  full_name='wandb_internal.ArtifactManifestEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='wandb_internal.ArtifactManifestEntry.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='digest', full_name='wandb_internal.ArtifactManifestEntry.digest', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ref', full_name='wandb_internal.ArtifactManifestEntry.ref', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='wandb_internal.ArtifactManifestEntry.size', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mimetype', full_name='wandb_internal.ArtifactManifestEntry.mimetype', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='local_path', full_name='wandb_internal.ArtifactManifestEntry.local_path', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extra', full_name='wandb_internal.ArtifactManifestEntry.extra', index=6,
      number=16, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2727,
  serialized_end=2887,
)


_EXTRAITEM = _descriptor.Descriptor(
  name='ExtraItem',
  full_name='wandb_internal.ExtraItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='wandb_internal.ExtraItem.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_json', full_name='wandb_internal.ExtraItem.value_json', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2889,
  serialized_end=2933,
)


_STORAGEPOLICYCONFIGITEM = _descriptor.Descriptor(
  name='StoragePolicyConfigItem',
  full_name='wandb_internal.StoragePolicyConfigItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='wandb_internal.StoragePolicyConfigItem.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_json', full_name='wandb_internal.StoragePolicyConfigItem.value_json', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2935,
  serialized_end=2993,
)


_EXITDATA = _descriptor.Descriptor(
  name='ExitData',
  full_name='wandb_internal.ExitData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='exit_code', full_name='wandb_internal.ExitData.exit_code', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2995,
  serialized_end=3024,
)


_STATUSDATA = _descriptor.Descriptor(
  name='StatusData',
  full_name='wandb_internal.StatusData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3026,
  serialized_end=3038,
)


_TBDATA = _descriptor.Descriptor(
  name='TBData',
  full_name='wandb_internal.TBData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='log_dir', full_name='wandb_internal.TBData.log_dir', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='save', full_name='wandb_internal.TBData.save', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3040,
  serialized_end=3079,
)


_RUNUPDATERESULT = _descriptor.Descriptor(
  name='RunUpdateResult',
  full_name='wandb_internal.RunUpdateResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run', full_name='wandb_internal.RunUpdateResult.run', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3081,
  serialized_end=3136,
)


_RUNEXITRESULT = _descriptor.Descriptor(
  name='RunExitResult',
  full_name='wandb_internal.RunExitResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3138,
  serialized_end=3153,
)


_LOGRESULT = _descriptor.Descriptor(
  name='LogResult',
  full_name='wandb_internal.LogResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3155,
  serialized_end=3166,
)


_STATUSRESULT = _descriptor.Descriptor(
  name='StatusResult',
  full_name='wandb_internal.StatusResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3168,
  serialized_end=3182,
)


_SUMMARYRESULT = _descriptor.Descriptor(
  name='SummaryResult',
  full_name='wandb_internal.SummaryResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3184,
  serialized_end=3199,
)


_OUTPUTRESULT = _descriptor.Descriptor(
  name='OutputResult',
  full_name='wandb_internal.OutputResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3201,
  serialized_end=3215,
)


_CONFIGRESULT = _descriptor.Descriptor(
  name='ConfigResult',
  full_name='wandb_internal.ConfigResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3217,
  serialized_end=3231,
)


_RESULTRECORD = _descriptor.Descriptor(
  name='ResultRecord',
  full_name='wandb_internal.ResultRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run_result', full_name='wandb_internal.ResultRecord.run_result', index=0,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exit_result', full_name='wandb_internal.ResultRecord.exit_result', index=1,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status_result', full_name='wandb_internal.ResultRecord.status_result', index=2,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log_result', full_name='wandb_internal.ResultRecord.log_result', index=3,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='summary_result', full_name='wandb_internal.ResultRecord.summary_result', index=4,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='output_result', full_name='wandb_internal.ResultRecord.output_result', index=5,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config_result', full_name='wandb_internal.ResultRecord.config_result', index=6,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='wandb_internal.ResultRecord.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=3234,
  serialized_end=3636,
)

_RECORD.fields_by_name['history'].message_type = _HISTORYDATA
_RECORD.fields_by_name['summary'].message_type = _SUMMARYDATA
_RECORD.fields_by_name['output'].message_type = _OUTPUTDATA
_RECORD.fields_by_name['config'].message_type = _CONFIGDATA
_RECORD.fields_by_name['files'].message_type = _FILESDATA
_RECORD.fields_by_name['stats'].message_type = _STATSDATA
_RECORD.fields_by_name['artifact'].message_type = _ARTIFACTDATA
_RECORD.fields_by_name['run'].message_type = _RUNDATA
_RECORD.fields_by_name['exit'].message_type = _EXITDATA
_RECORD.fields_by_name['status'].message_type = _STATUSDATA
_RECORD.fields_by_name['tbdata'].message_type = _TBDATA
_RECORD.fields_by_name['control'].message_type = _CONTROL
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['history'])
_RECORD.fields_by_name['history'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['summary'])
_RECORD.fields_by_name['summary'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['output'])
_RECORD.fields_by_name['output'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['config'])
_RECORD.fields_by_name['config'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['files'])
_RECORD.fields_by_name['files'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['stats'])
_RECORD.fields_by_name['stats'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['artifact'])
_RECORD.fields_by_name['artifact'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['run'])
_RECORD.fields_by_name['run'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['exit'])
_RECORD.fields_by_name['exit'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['status'])
_RECORD.fields_by_name['status'].containing_oneof = _RECORD.oneofs_by_name['data']
_RECORD.oneofs_by_name['data'].fields.append(
  _RECORD.fields_by_name['tbdata'])
_RECORD.fields_by_name['tbdata'].containing_oneof = _RECORD.oneofs_by_name['data']
_RUNDATA.fields_by_name['config'].message_type = _CONFIGDATA
_RUNDATA.fields_by_name['summary'].message_type = _SUMMARYDATA
_RUNDATA.fields_by_name['settings'].message_type = _SETTINGSDATA
_RUNDATA.fields_by_name['start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SETTINGSDATA.fields_by_name['item'].message_type = _SETTINGSITEM
_HISTORYDATA.fields_by_name['item'].message_type = _HISTORYITEM
_OUTPUTDATA.fields_by_name['output_type'].enum_type = _OUTPUTDATA_OUTPUTTYPE
_OUTPUTDATA.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_OUTPUTDATA_OUTPUTTYPE.containing_type = _OUTPUTDATA
_CONFIGDATA.fields_by_name['update'].message_type = _CONFIGITEM
_CONFIGDATA.fields_by_name['remove'].message_type = _CONFIGITEM
_SUMMARYDATA.fields_by_name['update'].message_type = _SUMMARYITEM
_SUMMARYDATA.fields_by_name['remove'].message_type = _SUMMARYITEM
_FILESDATA.fields_by_name['files'].message_type = _FILESITEM
_FILESITEM.fields_by_name['policy'].enum_type = _FILESITEM_POLICYTYPE
_FILESITEM_POLICYTYPE.containing_type = _FILESITEM
_STATSDATA.fields_by_name['stats_type'].enum_type = _STATSDATA_STATSTYPE
_STATSDATA.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STATSDATA.fields_by_name['item'].message_type = _STATSITEM
_STATSDATA_STATSTYPE.containing_type = _STATSDATA
_ARTIFACTDATA.fields_by_name['manifest'].message_type = _ARTIFACTMANIFEST
_ARTIFACTMANIFEST.fields_by_name['storage_policy_config'].message_type = _STORAGEPOLICYCONFIGITEM
_ARTIFACTMANIFEST.fields_by_name['contents'].message_type = _ARTIFACTMANIFESTENTRY
_ARTIFACTMANIFESTENTRY.fields_by_name['extra'].message_type = _EXTRAITEM
_RUNUPDATERESULT.fields_by_name['run'].message_type = _RUNDATA
_RESULTRECORD.fields_by_name['run_result'].message_type = _RUNUPDATERESULT
_RESULTRECORD.fields_by_name['exit_result'].message_type = _RUNEXITRESULT
_RESULTRECORD.fields_by_name['status_result'].message_type = _STATUSRESULT
_RESULTRECORD.fields_by_name['log_result'].message_type = _LOGRESULT
_RESULTRECORD.fields_by_name['summary_result'].message_type = _SUMMARYRESULT
_RESULTRECORD.fields_by_name['output_result'].message_type = _OUTPUTRESULT
_RESULTRECORD.fields_by_name['config_result'].message_type = _CONFIGRESULT
_RESULTRECORD.oneofs_by_name['data'].fields.append(
  _RESULTRECORD.fields_by_name['run_result'])
_RESULTRECORD.fields_by_name['run_result'].containing_oneof = _RESULTRECORD.oneofs_by_name['data']
_RESULTRECORD.oneofs_by_name['data'].fields.append(
  _RESULTRECORD.fields_by_name['exit_result'])
_RESULTRECORD.fields_by_name['exit_result'].containing_oneof = _RESULTRECORD.oneofs_by_name['data']
_RESULTRECORD.oneofs_by_name['data'].fields.append(
  _RESULTRECORD.fields_by_name['status_result'])
_RESULTRECORD.fields_by_name['status_result'].containing_oneof = _RESULTRECORD.oneofs_by_name['data']
_RESULTRECORD.oneofs_by_name['data'].fields.append(
  _RESULTRECORD.fields_by_name['log_result'])
_RESULTRECORD.fields_by_name['log_result'].containing_oneof = _RESULTRECORD.oneofs_by_name['data']
_RESULTRECORD.oneofs_by_name['data'].fields.append(
  _RESULTRECORD.fields_by_name['summary_result'])
_RESULTRECORD.fields_by_name['summary_result'].containing_oneof = _RESULTRECORD.oneofs_by_name['data']
_RESULTRECORD.oneofs_by_name['data'].fields.append(
  _RESULTRECORD.fields_by_name['output_result'])
_RESULTRECORD.fields_by_name['output_result'].containing_oneof = _RESULTRECORD.oneofs_by_name['data']
_RESULTRECORD.oneofs_by_name['data'].fields.append(
  _RESULTRECORD.fields_by_name['config_result'])
_RESULTRECORD.fields_by_name['config_result'].containing_oneof = _RESULTRECORD.oneofs_by_name['data']
DESCRIPTOR.message_types_by_name['Record'] = _RECORD
DESCRIPTOR.message_types_by_name['Control'] = _CONTROL
DESCRIPTOR.message_types_by_name['RunData'] = _RUNDATA
DESCRIPTOR.message_types_by_name['SettingsData'] = _SETTINGSDATA
DESCRIPTOR.message_types_by_name['SettingsItem'] = _SETTINGSITEM
DESCRIPTOR.message_types_by_name['HistoryData'] = _HISTORYDATA
DESCRIPTOR.message_types_by_name['HistoryItem'] = _HISTORYITEM
DESCRIPTOR.message_types_by_name['OutputData'] = _OUTPUTDATA
DESCRIPTOR.message_types_by_name['ConfigData'] = _CONFIGDATA
DESCRIPTOR.message_types_by_name['ConfigItem'] = _CONFIGITEM
DESCRIPTOR.message_types_by_name['SummaryData'] = _SUMMARYDATA
DESCRIPTOR.message_types_by_name['SummaryItem'] = _SUMMARYITEM
DESCRIPTOR.message_types_by_name['FilesData'] = _FILESDATA
DESCRIPTOR.message_types_by_name['FilesItem'] = _FILESITEM
DESCRIPTOR.message_types_by_name['StatsData'] = _STATSDATA
DESCRIPTOR.message_types_by_name['StatsItem'] = _STATSITEM
DESCRIPTOR.message_types_by_name['ArtifactData'] = _ARTIFACTDATA
DESCRIPTOR.message_types_by_name['ArtifactManifest'] = _ARTIFACTMANIFEST
DESCRIPTOR.message_types_by_name['ArtifactManifestEntry'] = _ARTIFACTMANIFESTENTRY
DESCRIPTOR.message_types_by_name['ExtraItem'] = _EXTRAITEM
DESCRIPTOR.message_types_by_name['StoragePolicyConfigItem'] = _STORAGEPOLICYCONFIGITEM
DESCRIPTOR.message_types_by_name['ExitData'] = _EXITDATA
DESCRIPTOR.message_types_by_name['StatusData'] = _STATUSDATA
DESCRIPTOR.message_types_by_name['TBData'] = _TBDATA
DESCRIPTOR.message_types_by_name['RunUpdateResult'] = _RUNUPDATERESULT
DESCRIPTOR.message_types_by_name['RunExitResult'] = _RUNEXITRESULT
DESCRIPTOR.message_types_by_name['LogResult'] = _LOGRESULT
DESCRIPTOR.message_types_by_name['StatusResult'] = _STATUSRESULT
DESCRIPTOR.message_types_by_name['SummaryResult'] = _SUMMARYRESULT
DESCRIPTOR.message_types_by_name['OutputResult'] = _OUTPUTRESULT
DESCRIPTOR.message_types_by_name['ConfigResult'] = _CONFIGRESULT
DESCRIPTOR.message_types_by_name['ResultRecord'] = _RESULTRECORD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Record = _reflection.GeneratedProtocolMessageType('Record', (_message.Message,), {
  'DESCRIPTOR' : _RECORD,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.Record)
  })
_sym_db.RegisterMessage(Record)

Control = _reflection.GeneratedProtocolMessageType('Control', (_message.Message,), {
  'DESCRIPTOR' : _CONTROL,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.Control)
  })
_sym_db.RegisterMessage(Control)

RunData = _reflection.GeneratedProtocolMessageType('RunData', (_message.Message,), {
  'DESCRIPTOR' : _RUNDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.RunData)
  })
_sym_db.RegisterMessage(RunData)

SettingsData = _reflection.GeneratedProtocolMessageType('SettingsData', (_message.Message,), {
  'DESCRIPTOR' : _SETTINGSDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.SettingsData)
  })
_sym_db.RegisterMessage(SettingsData)

SettingsItem = _reflection.GeneratedProtocolMessageType('SettingsItem', (_message.Message,), {
  'DESCRIPTOR' : _SETTINGSITEM,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.SettingsItem)
  })
_sym_db.RegisterMessage(SettingsItem)

HistoryData = _reflection.GeneratedProtocolMessageType('HistoryData', (_message.Message,), {
  'DESCRIPTOR' : _HISTORYDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.HistoryData)
  })
_sym_db.RegisterMessage(HistoryData)

HistoryItem = _reflection.GeneratedProtocolMessageType('HistoryItem', (_message.Message,), {
  'DESCRIPTOR' : _HISTORYITEM,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.HistoryItem)
  })
_sym_db.RegisterMessage(HistoryItem)

OutputData = _reflection.GeneratedProtocolMessageType('OutputData', (_message.Message,), {
  'DESCRIPTOR' : _OUTPUTDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.OutputData)
  })
_sym_db.RegisterMessage(OutputData)

ConfigData = _reflection.GeneratedProtocolMessageType('ConfigData', (_message.Message,), {
  'DESCRIPTOR' : _CONFIGDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ConfigData)
  })
_sym_db.RegisterMessage(ConfigData)

ConfigItem = _reflection.GeneratedProtocolMessageType('ConfigItem', (_message.Message,), {
  'DESCRIPTOR' : _CONFIGITEM,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ConfigItem)
  })
_sym_db.RegisterMessage(ConfigItem)

SummaryData = _reflection.GeneratedProtocolMessageType('SummaryData', (_message.Message,), {
  'DESCRIPTOR' : _SUMMARYDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.SummaryData)
  })
_sym_db.RegisterMessage(SummaryData)

SummaryItem = _reflection.GeneratedProtocolMessageType('SummaryItem', (_message.Message,), {
  'DESCRIPTOR' : _SUMMARYITEM,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.SummaryItem)
  })
_sym_db.RegisterMessage(SummaryItem)

FilesData = _reflection.GeneratedProtocolMessageType('FilesData', (_message.Message,), {
  'DESCRIPTOR' : _FILESDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.FilesData)
  })
_sym_db.RegisterMessage(FilesData)

FilesItem = _reflection.GeneratedProtocolMessageType('FilesItem', (_message.Message,), {
  'DESCRIPTOR' : _FILESITEM,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.FilesItem)
  })
_sym_db.RegisterMessage(FilesItem)

StatsData = _reflection.GeneratedProtocolMessageType('StatsData', (_message.Message,), {
  'DESCRIPTOR' : _STATSDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.StatsData)
  })
_sym_db.RegisterMessage(StatsData)

StatsItem = _reflection.GeneratedProtocolMessageType('StatsItem', (_message.Message,), {
  'DESCRIPTOR' : _STATSITEM,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.StatsItem)
  })
_sym_db.RegisterMessage(StatsItem)

ArtifactData = _reflection.GeneratedProtocolMessageType('ArtifactData', (_message.Message,), {
  'DESCRIPTOR' : _ARTIFACTDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ArtifactData)
  })
_sym_db.RegisterMessage(ArtifactData)

ArtifactManifest = _reflection.GeneratedProtocolMessageType('ArtifactManifest', (_message.Message,), {
  'DESCRIPTOR' : _ARTIFACTMANIFEST,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ArtifactManifest)
  })
_sym_db.RegisterMessage(ArtifactManifest)

ArtifactManifestEntry = _reflection.GeneratedProtocolMessageType('ArtifactManifestEntry', (_message.Message,), {
  'DESCRIPTOR' : _ARTIFACTMANIFESTENTRY,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ArtifactManifestEntry)
  })
_sym_db.RegisterMessage(ArtifactManifestEntry)

ExtraItem = _reflection.GeneratedProtocolMessageType('ExtraItem', (_message.Message,), {
  'DESCRIPTOR' : _EXTRAITEM,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ExtraItem)
  })
_sym_db.RegisterMessage(ExtraItem)

StoragePolicyConfigItem = _reflection.GeneratedProtocolMessageType('StoragePolicyConfigItem', (_message.Message,), {
  'DESCRIPTOR' : _STORAGEPOLICYCONFIGITEM,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.StoragePolicyConfigItem)
  })
_sym_db.RegisterMessage(StoragePolicyConfigItem)

ExitData = _reflection.GeneratedProtocolMessageType('ExitData', (_message.Message,), {
  'DESCRIPTOR' : _EXITDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ExitData)
  })
_sym_db.RegisterMessage(ExitData)

StatusData = _reflection.GeneratedProtocolMessageType('StatusData', (_message.Message,), {
  'DESCRIPTOR' : _STATUSDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.StatusData)
  })
_sym_db.RegisterMessage(StatusData)

TBData = _reflection.GeneratedProtocolMessageType('TBData', (_message.Message,), {
  'DESCRIPTOR' : _TBDATA,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.TBData)
  })
_sym_db.RegisterMessage(TBData)

RunUpdateResult = _reflection.GeneratedProtocolMessageType('RunUpdateResult', (_message.Message,), {
  'DESCRIPTOR' : _RUNUPDATERESULT,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.RunUpdateResult)
  })
_sym_db.RegisterMessage(RunUpdateResult)

RunExitResult = _reflection.GeneratedProtocolMessageType('RunExitResult', (_message.Message,), {
  'DESCRIPTOR' : _RUNEXITRESULT,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.RunExitResult)
  })
_sym_db.RegisterMessage(RunExitResult)

LogResult = _reflection.GeneratedProtocolMessageType('LogResult', (_message.Message,), {
  'DESCRIPTOR' : _LOGRESULT,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.LogResult)
  })
_sym_db.RegisterMessage(LogResult)

StatusResult = _reflection.GeneratedProtocolMessageType('StatusResult', (_message.Message,), {
  'DESCRIPTOR' : _STATUSRESULT,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.StatusResult)
  })
_sym_db.RegisterMessage(StatusResult)

SummaryResult = _reflection.GeneratedProtocolMessageType('SummaryResult', (_message.Message,), {
  'DESCRIPTOR' : _SUMMARYRESULT,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.SummaryResult)
  })
_sym_db.RegisterMessage(SummaryResult)

OutputResult = _reflection.GeneratedProtocolMessageType('OutputResult', (_message.Message,), {
  'DESCRIPTOR' : _OUTPUTRESULT,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.OutputResult)
  })
_sym_db.RegisterMessage(OutputResult)

ConfigResult = _reflection.GeneratedProtocolMessageType('ConfigResult', (_message.Message,), {
  'DESCRIPTOR' : _CONFIGRESULT,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ConfigResult)
  })
_sym_db.RegisterMessage(ConfigResult)

ResultRecord = _reflection.GeneratedProtocolMessageType('ResultRecord', (_message.Message,), {
  'DESCRIPTOR' : _RESULTRECORD,
  '__module__' : 'wandb.proto.wandb_internal_pb2'
  # @@protoc_insertion_point(class_scope:wandb_internal.ResultRecord)
  })
_sym_db.RegisterMessage(ResultRecord)


# @@protoc_insertion_point(module_scope)
